import request from 'supertest'
import { generateSimpleViteManifest, mockConfig, mockFetch, mockApp, mockRedis } from './util.js'
import { expect } from 'chai'
import * as td from 'testdouble'
import RedisMock from 'ioredis-mock'

describe('Redirects', function () {
  let app

  before(async function () {
    mockConfig({ baseUrls: ['http://ui-server/'] })
    mockRedis()
    mockFetch({
      'http://ui-server': {
        '/manifest.json': generateSimpleViteManifest({ 'example.js': 'test' }),
        '/example.js': ''
      }
    })
    app = await mockApp()
  })

  afterEach(async function () {
    await new RedisMock().flushdb()
  })

  after(function () {
    td.reset()
  })

  it('without requested location', async function () {
    const response = await request(app.server).post('/redirect')
    expect(response.statusCode).to.equal(302)
    expect(response.headers.location).to.equal('../busy.html')
  })

  it('with requested location', async function () {
    const response = await request(app.server).post('/redirect').send('location=/appsuite/whatever/path')
    expect(response.statusCode).to.equal(302)
    expect(response.headers.location).to.equal('/appsuite/whatever/path')
  })

  it('redirects /ui to /', async function () {
    const response = await request(app.server).get('/ui')
    expect(response.statusCode).to.equal(302)
    expect(response.headers.location).to.equal('/')
  })
})
